@@ -126,6 +126,11 @@ gboolean            nm_manager_deactivate_connection   (NMManager *manager,
 
 void                nm_manager_set_capability   (NMManager *self, NMCapability cap);
 
-gboolean            nm_manager_remove_device (NMManager *self, const char *ifname);
+NMDevice *          nm_manager_get_device    (NMManager *self,
+                                              const char *ifname,
+                                              NMDeviceType device_type);
+gboolean            nm_manager_remove_device (NMManager *self,
+                                              const char *ifname,
+                                              NMDeviceType device_type);
 
 #endif /* __NETWORKMANAGER_MANAGER_H__ */
