@@ -226,6 +226,7 @@ write_object (NMSetting8021x *s_8021x,
 	NMSettingSecretFlags flags = NM_SETTING_SECRET_FLAG_NONE;
 	char *secret_name, *secret_flags;
 	const char *extension;
+	char *standard_file;
 
 	g_return_val_if_fail (ifcfg != NULL, FALSE);
 	g_return_val_if_fail (objtype != NULL, FALSE);
@@ -265,24 +266,6 @@ write_object (NMSetting8021x *s_8021x,
 	else
 		extension = "pem";
 
-	/* If certificate/private key wasn't sent, the connection may no longer be
-	 * 802.1x and thus we clear out the paths and certs.
-	 */
-	if (!value && !blob) {
-		char *standard_file;
-
-		/* Since no cert/private key is now being used, delete any standard file
-		 * that was created for this connection, but leave other files alone.
-		 * Thus, for example,
-		 * /etc/sysconfig/network-scripts/ca-cert-Test_Write_Wifi_WPA_EAP-TLS.der
-		 * will be deleted, but /etc/pki/tls/cert.pem will not.
-		 */
-		standard_file = utils_cert_path (svFileGetName (ifcfg), objtype->vtable->file_suffix, extension);
-		g_hash_table_replace (blobs, standard_file, NULL);
-		svUnsetValue (ifcfg, objtype->ifcfg_rh_key);
-		return TRUE;
-	}
-
 	/* If the object path was specified, prefer that over any raw cert data that
 	 * may have been sent.
 	 */
@@ -301,6 +284,18 @@ write_object (NMSetting8021x *s_8021x,
 		return TRUE;
 	}
 
+	/* If certificate/private key wasn't sent, the connection may no longer be
+	 * 802.1x and thus we clear out the paths and certs.
+	 *
+	 * Since no cert/private key is now being used, delete any standard file
+	 * that was created for this connection, but leave other files alone.
+	 * Thus, for example,
+	 * /etc/sysconfig/network-scripts/ca-cert-Test_Write_Wifi_WPA_EAP-TLS.der
+	 * will be deleted, but /etc/pki/tls/cert.pem will not.
+	 */
+	standard_file = utils_cert_path (svFileGetName (ifcfg), objtype->vtable->file_suffix, extension);
+	g_hash_table_replace (blobs, standard_file, NULL);
+	svUnsetValue (ifcfg, objtype->ifcfg_rh_key);
 	return TRUE;
 }
 
@@ -403,11 +398,14 @@ write_8021x_setting (NMConnection *connection,
 	NMSetting8021x *s_8021x;
 	NMSetting8021xAuthFlags auth_flags;
 	const char *value, *match;
+	gconstpointer ptr;
+	GBytes* bytes;
 	char *tmp = NULL;
 	GString *phase2_auth;
 	GString *str;
 	guint32 i, num;
-	gint timeout;
+	gsize size;
+	int vint;
 
 	s_8021x = nm_connection_get_setting_802_1x (connection);
 	if (!s_8021x) {
@@ -443,6 +441,20 @@ write_8021x_setting (NMConnection *connection,
 	            "IEEE_8021X_PASSWORD_FLAGS",
 	            nm_setting_802_1x_get_password_flags (s_8021x));
 
+	tmp = NULL;
+	bytes = nm_setting_802_1x_get_password_raw (s_8021x);
+	if (bytes) {
+		ptr = g_bytes_get_data (bytes, &size);
+		tmp = nm_utils_bin2hexstr (ptr, size, -1);
+	}
+	set_secret (ifcfg,
+	            secrets,
+	            "IEEE_8021X_PASSWORD_RAW",
+	            tmp,
+	            "IEEE_8021X_PASSWORD_RAW_FLAGS",
+	            nm_setting_802_1x_get_password_raw_flags (s_8021x));
+	g_free (tmp);
+
 	/* PEAP version */
 	value = nm_setting_802_1x_get_phase1_peapver (s_8021x);
 	svUnsetValue (ifcfg, "IEEE_8021X_PEAP_VERSION");
@@ -545,11 +557,8 @@ write_8021x_setting (NMConnection *connection,
 	svSetValueStr (ifcfg, "IEEE_8021X_PHASE2_DOMAIN_SUFFIX_MATCH",
 	               nm_setting_802_1x_get_phase2_domain_suffix_match (s_8021x));
 
-	timeout = nm_setting_802_1x_get_auth_timeout (s_8021x);
-	if (timeout > 0)
-		svSetValueInt64 (ifcfg, "IEEE_8021X_AUTH_TIMEOUT", timeout);
-	else
-		svUnsetValue (ifcfg, "IEEE_8021X_AUTH_TIMEOUT");
+	vint = nm_setting_802_1x_get_auth_timeout (s_8021x);
+	svSetValueInt64_cond (ifcfg, "IEEE_8021X_AUTH_TIMEOUT", vint > 0, vint);
 
 	if (!write_8021x_certs (s_8021x, secrets, blobs, FALSE, ifcfg, error))
 		return FALSE;
@@ -688,7 +697,7 @@ write_wireless_security_setting (NMConnection *connection,
 						key = ascii_key;
 					}
 				} else {
-					_LOGW ("invalid WEP key '%s'", key);
+					g_warn_if_reached ();
 					key_valid = FALSE;
 				}
 
@@ -1385,7 +1394,7 @@ get_setting_default_boolean (NMSetting *setting, const char *prop)
 }
 
 static gboolean
-write_bridge_setting (NMConnection *connection, shvarFile *ifcfg, GError **error)
+write_bridge_setting (NMConnection *connection, shvarFile *ifcfg, gboolean *wired, GError **error)
 {
 	NMSettingBridge *s_bridge;
 	guint32 i;
@@ -1461,6 +1470,8 @@ write_bridge_setting (NMConnection *connection, shvarFile *ifcfg, GError **error
 
 	svSetValueStr (ifcfg, "TYPE", TYPE_BRIDGE);
 
+	*wired = write_wired_for_virtual (connection, ifcfg);
+
 	return TRUE;
 }
 
@@ -1709,7 +1720,8 @@ write_connection_setting (NMSettingConnection *s_con, shvarFile *ifcfg)
 	guint32 n, i;
 	GString *str;
 	const char *master, *master_iface = NULL, *type;
-	gint i_int;
+	gint vint;
+	guint32 vuint32;
 	const char *tmp;
 
 	svSetValueStr (ifcfg, "NAME", nm_setting_connection_get_id (s_con));
@@ -1718,17 +1730,15 @@ write_connection_setting (NMSettingConnection *s_con, shvarFile *ifcfg)
 	svSetValueStr (ifcfg, "DEVICE", nm_setting_connection_get_interface_name (s_con));
 	svSetValueBoolean (ifcfg, "ONBOOT", nm_setting_connection_get_autoconnect (s_con));
 
-	i_int = nm_setting_connection_get_autoconnect_priority (s_con);
-	if (i_int != NM_SETTING_CONNECTION_AUTOCONNECT_PRIORITY_DEFAULT)
-		svSetValueInt64 (ifcfg, "AUTOCONNECT_PRIORITY", i_int);
-	else
-		svUnsetValue (ifcfg, "AUTOCONNECT_PRIORITY");
+	vint = nm_setting_connection_get_autoconnect_priority (s_con);
+	svSetValueInt64_cond (ifcfg, "AUTOCONNECT_PRIORITY",
+	                      vint != NM_SETTING_CONNECTION_AUTOCONNECT_PRIORITY_DEFAULT,
+	                      vint);
 
-	i_int = nm_setting_connection_get_autoconnect_retries (s_con);
-	if (i_int != -1)
-		svSetValueInt64 (ifcfg, "AUTOCONNECT_RETRIES", i_int);
-	else
-		svUnsetValue (ifcfg, "AUTOCONNECT_RETRIES");
+	vint = nm_setting_connection_get_autoconnect_retries (s_con);
+	svSetValueInt64_cond (ifcfg, "AUTOCONNECT_RETRIES",
+	                      vint != -1,
+	                      vint);
 
 	/* Only save the value for master connections */
 	type = nm_setting_connection_get_connection_type (s_con);
@@ -1844,11 +1854,10 @@ write_connection_setting (NMSettingConnection *s_con, shvarFile *ifcfg)
 		g_string_free (str, TRUE);
 	}
 
-	if (nm_setting_connection_get_gateway_ping_timeout (s_con)) {
-		svSetValueInt64 (ifcfg, "GATEWAY_PING_TIMEOUT",
-		                 nm_setting_connection_get_gateway_ping_timeout (s_con));
-	} else
-		svUnsetValue (ifcfg, "GATEWAY_PING_TIMEOUT");
+	vuint32 = nm_setting_connection_get_gateway_ping_timeout (s_con);
+	svSetValueInt64_cond (ifcfg, "GATEWAY_PING_TIMEOUT",
+	                      vuint32 != 0,
+	                      vuint32);
 
 	switch (nm_setting_connection_get_metered (s_con)) {
 	case NM_METERED_YES:
@@ -1860,23 +1869,26 @@ write_connection_setting (NMSettingConnection *s_con, shvarFile *ifcfg)
 	default:
 		svUnsetValue (ifcfg, "CONNECTION_METERED");
 	}
+
+	vint = nm_setting_connection_get_auth_retries (s_con);
+	svSetValueInt64_cond (ifcfg, "AUTH_RETRIES", vint >= 0, vint);
 }
 
 static char *
 get_route_attributes_string (NMIPRoute *route, int family)
 {
-	gs_strfreev char **names = NULL;
+	gs_free const char **names = NULL;
 	GVariant *attr, *lock;
 	GString *str;
-	int i;
+	guint i, len;
 
-	names = nm_ip_route_get_attribute_names (route);
-	if (!names || !names[0])
+	names = _nm_ip_route_get_attribute_names (route, TRUE, &len);
+	if (!len)
 		return NULL;
 
 	str = g_string_new ("");
 
-	for (i = 0; names[i]; i++) {
+	for (i = 0; i < len; i++) {
 		attr = nm_ip_route_get_attribute (route, names[i]);
 
 		if (!nm_ip_route_attribute_validate (names[i], attr, family, NULL, NULL))
@@ -1919,7 +1931,7 @@ get_route_attributes_string (NMIPRoute *route, int family)
 
 			g_string_append_printf (str, "%s %s", arg, g_variant_get_string (attr, NULL));
 		} else {
-			_LOGW ("unknown route option '%s'", names[i]);
+			g_warn_if_reached ();
 			continue;
 		}
 		if (names[i + 1])
@@ -2331,7 +2343,7 @@ write_ip4_setting (NMConnection *connection,
 }
 
 static void
-write_ip4_aliases (NMConnection *connection, char *base_ifcfg_path)
+write_ip4_aliases (NMConnection *connection, const char *base_ifcfg_path)
 {
 	NMSettingIPConfig *s_ip4;
 	gs_free char *base_ifcfg_dir = NULL, *base_ifcfg_name = NULL;
@@ -2705,42 +2717,44 @@ escape_id (const char *id)
 	return escaped;
 }
 
-gboolean
-nms_ifcfg_rh_writer_write_connection (NMConnection *connection,
-                                      const char *ifcfg_dir,
-                                      const char *filename,
-                                      char **out_filename,
-                                      NMConnection **out_reread,
-                                      gboolean *out_reread_same,
-                                      GError **error)
+static gboolean
+do_write_construct (NMConnection *connection,
+                    const char *ifcfg_dir,
+                    const char *filename,
+                    shvarFile **out_ifcfg,
+                    GHashTable **out_blobs,
+                    GHashTable **out_secrets,
+                    gboolean *out_route_ignore,
+                    shvarFile **out_route_content_svformat,
+                    GString **out_route_content,
+                    GString **out_route6_content,
+                    GError **error)
 {
 	NMSettingConnection *s_con;
 	nm_auto_shvar_file_close shvarFile *ifcfg = NULL;
 	gs_free char *ifcfg_name = NULL;
-	const char *type;
-	gboolean no_8021x = FALSE;
-	gboolean wired = FALSE;
-	gboolean route_path_is_svformat;
 	gs_free char *route_path = NULL;
 	gs_free char *route6_path = NULL;
-	nm_auto_free_gstring GString *route_content = NULL;
-	gboolean route_ignore = FALSE;
+	const char *type;
+	gs_unref_hashtable GHashTable *blobs = NULL;
+	gs_unref_hashtable GHashTable *secrets = NULL;
+	gboolean wired;
+	gboolean no_8021x;
+	gboolean route_path_is_svformat;
 	gboolean has_complex_routes_v4;
 	gboolean has_complex_routes_v6;
+	gboolean route_ignore;
 	nm_auto_shvar_file_close shvarFile *route_content_svformat = NULL;
+	nm_auto_free_gstring GString *route_content = NULL;
 	nm_auto_free_gstring GString *route6_content = NULL;
-	gs_unref_hashtable GHashTable *secrets = NULL;
-	gs_unref_hashtable GHashTable *blobs = NULL;
 
 	nm_assert (NM_IS_CONNECTION (connection));
 	nm_assert (_nm_connection_verify (connection, NULL) == NM_SETTING_VERIFY_SUCCESS);
-	nm_assert (!out_reread || !*out_reread);
 
 	if (!nms_ifcfg_rh_writer_can_write_connection (connection, error))
 		return FALSE;
 
 	s_con = nm_connection_get_setting_connection (connection);
-	g_assert (s_con);
 
 	if (filename) {
 		/* For existing connections, 'filename' should be full path to ifcfg file */
@@ -2749,7 +2763,7 @@ nms_ifcfg_rh_writer_write_connection (NMConnection *connection,
 			return FALSE;
 
 		ifcfg_name = g_strdup (filename);
-	} else {
+	} else if (ifcfg_dir) {
 		char *escaped;
 
 		escaped = escape_id (nm_setting_connection_get_id (s_con));
@@ -2780,7 +2794,8 @@ nms_ifcfg_rh_writer_write_connection (NMConnection *connection,
 		}
 
 		ifcfg = svCreateFile (ifcfg_name);
-	}
+	} else
+		ifcfg = svCreateFile ("/tmp/ifcfg-dummy");
 
 	route_path = utils_get_route_path (svFileGetName (ifcfg));
 	if (!route_path) {
@@ -2805,6 +2820,8 @@ nms_ifcfg_rh_writer_write_connection (NMConnection *connection,
 
 	secrets = g_hash_table_new_full (nm_str_hash, g_str_equal, g_free, g_free);
 
+	wired = FALSE;
+	no_8021x = FALSE;
 	if (!strcmp (type, NM_SETTING_WIRED_SETTING_NAME)) {
 		// FIXME: can't write PPPoE at this time
 		if (nm_connection_get_setting_pppoe (connection)) {
@@ -2833,7 +2850,7 @@ nms_ifcfg_rh_writer_write_connection (NMConnection *connection,
 		if (!write_team_setting (connection, ifcfg, &wired, error))
 			return FALSE;
 	} else if (!strcmp (type, NM_SETTING_BRIDGE_SETTING_NAME)) {
-		if (!write_bridge_setting (connection, ifcfg, error))
+		if (!write_bridge_setting (connection, ifcfg, &wired, error))
 			return FALSE;
 	} else {
 		g_set_error (error, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_FAILED,
@@ -2894,7 +2911,8 @@ nms_ifcfg_rh_writer_write_connection (NMConnection *connection,
 			return FALSE;
 		}
 		route_ignore = TRUE;
-	}
+	} else
+		route_ignore = FALSE;
 
 	if (!write_ip4_setting (connection,
 	                        ifcfg,
@@ -2914,6 +2932,27 @@ nms_ifcfg_rh_writer_write_connection (NMConnection *connection,
 
 	write_connection_setting (s_con, ifcfg);
 
+	NM_SET_OUT (out_ifcfg, g_steal_pointer (&ifcfg));
+	NM_SET_OUT (out_blobs, g_steal_pointer (&blobs));
+	NM_SET_OUT (out_secrets, g_steal_pointer (&secrets));
+	NM_SET_OUT (out_route_ignore, route_ignore);
+	NM_SET_OUT (out_route_content_svformat, g_steal_pointer (&route_content_svformat));
+	NM_SET_OUT (out_route_content, g_steal_pointer (&route_content));
+	NM_SET_OUT (out_route6_content, g_steal_pointer (&route6_content));
+	return TRUE;
+}
+
+static gboolean
+do_write_to_disk (NMConnection *connection,
+                  shvarFile *ifcfg,
+                  GHashTable *blobs,
+                  GHashTable *secrets,
+                  gboolean route_ignore,
+                  shvarFile *route_content_svformat,
+                  GString *route_content,
+                  GString *route6_content,
+                  GError **error)
+{
 	/* From here on, we persist data to disk. Before, it was all in-memory
 	 * only. But we loaded the ifcfg files from disk, and managled our
 	 * new settings (in-momory). */
@@ -2921,7 +2960,7 @@ nms_ifcfg_rh_writer_write_connection (NMConnection *connection,
 	if (!svWriteFile (ifcfg, 0644, error))
 		return FALSE;
 
-	write_ip4_aliases (connection, ifcfg_name);
+	write_ip4_aliases (connection, svFileGetName (ifcfg));
 
 	if (!write_blobs (blobs, error))
 		return FALSE;
@@ -2930,10 +2969,13 @@ nms_ifcfg_rh_writer_write_connection (NMConnection *connection,
 		return FALSE;
 
 	if (!route_ignore) {
+		gs_free char *route_path = utils_get_route_path (svFileGetName (ifcfg));
+
 		if (!route_content && !route_content_svformat)
 			(void) unlink (route_path);
 		else {
-			if (route_path_is_svformat) {
+			nm_assert (route_content_svformat || route_content);
+			if (route_content_svformat) {
 				if (!svWriteFile (route_content_svformat, 0644, error))
 					return FALSE;
 			} else {
@@ -2947,6 +2989,8 @@ nms_ifcfg_rh_writer_write_connection (NMConnection *connection,
 	}
 
 	if (!route_ignore) {
+		gs_free char *route6_path = utils_get_route6_path (svFileGetName (ifcfg));
+
 		if (!route6_content)
 			(void) unlink (route6_path);
 		else {
@@ -2958,50 +3002,131 @@ nms_ifcfg_rh_writer_write_connection (NMConnection *connection,
 		}
 	}
 
+	return TRUE;
+}
+
+static gboolean
+do_write_reread (NMConnection *connection,
+                 const char *ifcfg_name,
+                 NMConnection **out_reread,
+                 gboolean *out_reread_same,
+                 GError **error)
+{
+	gs_unref_object NMConnection *reread = NULL;
+	gs_free_error GError *local = NULL;
+	gs_free char *unhandled = NULL;
+	gboolean reread_same = FALSE;
+
+	nm_assert (!out_reread || !*out_reread);
+
+	reread = connection_from_file (ifcfg_name, &unhandled, &local, NULL);
+
+	if (!reread) {
+		g_propagate_error (error, local);
+		local = NULL;
+		return FALSE;
+	}
+	if (unhandled) {
+		g_set_error (error, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_FAILED,
+		             "connection is unhandled");
+		return FALSE;
+	}
+	if (out_reread_same) {
+		if (nm_connection_compare (reread, connection, NM_SETTING_COMPARE_FLAG_EXACT))
+			reread_same = TRUE;
+
+		nm_assert (reread_same == nm_connection_compare (connection, reread, NM_SETTING_COMPARE_FLAG_EXACT));
+		nm_assert (reread_same == ({
+		                                gs_unref_hashtable GHashTable *_settings = NULL;
+
+		                                (   nm_connection_diff (reread, connection, NM_SETTING_COMPARE_FLAG_EXACT, &_settings)
+		                                 && !_settings);
+		                           }));
+	}
+
+	NM_SET_OUT (out_reread, g_steal_pointer (&reread));
+	NM_SET_OUT (out_reread_same, reread_same);
+	return TRUE;
+}
+
+gboolean
+nms_ifcfg_rh_writer_write_connection (NMConnection *connection,
+                                      const char *ifcfg_dir,
+                                      const char *filename,
+                                      char **out_filename,
+                                      NMConnection **out_reread,
+                                      gboolean *out_reread_same,
+                                      GError **error)
+{
+	nm_auto_shvar_file_close shvarFile *ifcfg = NULL;
+	nm_auto_free_gstring GString *route_content = NULL;
+	gboolean route_ignore = FALSE;
+	nm_auto_shvar_file_close shvarFile *route_content_svformat = NULL;
+	nm_auto_free_gstring GString *route6_content = NULL;
+	gs_unref_hashtable GHashTable *secrets = NULL;
+	gs_unref_hashtable GHashTable *blobs = NULL;
+	GError *local = NULL;
+
+	nm_assert (!out_reread || !*out_reread);
+
+	if (!do_write_construct (connection,
+	                         ifcfg_dir,
+	                         filename,
+	                         &ifcfg,
+	                         &blobs,
+	                         &secrets,
+	                         &route_ignore,
+	                         &route_content_svformat,
+	                         &route_content,
+	                         &route6_content,
+	                         error))
+		return FALSE;
+
+	_LOGT ("write: write connection %s (%s) to file \"%s\"",
+	       nm_connection_get_id (connection),
+	       nm_connection_get_uuid (connection),
+	       svFileGetName (ifcfg));
+
+	if (!do_write_to_disk (connection,
+	                       ifcfg,
+	                       blobs,
+	                       secrets,
+	                       route_ignore,
+	                       route_content_svformat,
+	                       route_content,
+	                       route6_content,
+	                       error))
+		return FALSE;
+
 	/* Note that we just wrote the connection to disk, and re-read it from there.
 	 * That is racy if somebody else modifies the connection.
 	 *
 	 * A better solution might be, to re-read the connection only based on the
 	 * in-memory representation of what we collected above. But the reader
 	 * does not yet allow to inject the configuration. */
-
 	if (out_reread || out_reread_same) {
-		gs_unref_object NMConnection *reread = NULL;
-		gs_free_error GError *local = NULL;
-		gs_free char *unhandled = NULL;
-		gboolean reread_same = FALSE;
-
-		reread = connection_from_file (ifcfg_name, &unhandled, &local, NULL);
-
-		if (unhandled) {
-			_LOGW ("failure to re-read the new connection from file \"%s\": %s",
-			       ifcfg_name, "connection is unhandled");
-			g_clear_object (&reread);
-		} else if (!reread) {
-			_LOGW ("failure to re-read the new connection from file \"%s\": %s",
-			       ifcfg_name, local ? local->message : "<unknown>");
+		if (!do_write_reread (connection,
+		                         svFileGetName (ifcfg),
+		                         out_reread,
+		                         out_reread_same,
+		                         &local)) {
+			_LOGW ("write: failure to re-read connection \"%s\": %s",
+			       svFileGetName (ifcfg), local->message);
+			g_clear_error (&local);
 		} else {
-			if (out_reread_same) {
-				if (nm_connection_compare (reread, connection, NM_SETTING_COMPARE_FLAG_EXACT))
-					reread_same = TRUE;
-
-				nm_assert (reread_same == nm_connection_compare (connection, reread, NM_SETTING_COMPARE_FLAG_EXACT));
-				nm_assert (reread_same == ({
-				                                gs_unref_hashtable GHashTable *_settings = NULL;
-
-				                                (   nm_connection_diff (reread, connection, NM_SETTING_COMPARE_FLAG_EXACT, &_settings)
-				                                 && !_settings);
-				                           }));
+			if (   out_reread_same
+			    && !*out_reread_same) {
+				_LOGD ("write: connection %s (%s) was modified by persisting it to \"%s\" ",
+				       nm_connection_get_id (connection),
+				       nm_connection_get_uuid (connection),
+				       svFileGetName (ifcfg));
 			}
 		}
-
-		NM_SET_OUT (out_reread, g_steal_pointer (&reread));
-		NM_SET_OUT (out_reread_same, reread_same);
 	}
 
 	/* Only return the filename if this was a newly written ifcfg */
 	if (out_filename && !filename)
-		*out_filename = g_steal_pointer (&ifcfg_name);
+		*out_filename = g_strdup (svFileGetName (ifcfg));
 
 	return TRUE;
 }
@@ -3009,25 +3134,26 @@ nms_ifcfg_rh_writer_write_connection (NMConnection *connection,
 gboolean
 nms_ifcfg_rh_writer_can_write_connection (NMConnection *connection, GError **error)
 {
-	NMSettingConnection *s_con;
+	const char *type, *id;
 
-	if (   (   nm_connection_is_type (connection, NM_SETTING_WIRED_SETTING_NAME)
-	        && !nm_connection_get_setting_pppoe (connection))
-	    || nm_connection_is_type (connection, NM_SETTING_VLAN_SETTING_NAME)
-	    || nm_connection_is_type (connection, NM_SETTING_WIRELESS_SETTING_NAME)
-	    || nm_connection_is_type (connection, NM_SETTING_INFINIBAND_SETTING_NAME)
-	    || nm_connection_is_type (connection, NM_SETTING_BOND_SETTING_NAME)
-	    || nm_connection_is_type (connection, NM_SETTING_TEAM_SETTING_NAME)
-	    || nm_connection_is_type (connection, NM_SETTING_BRIDGE_SETTING_NAME))
+	type = nm_connection_get_connection_type (connection);
+	if (NM_IN_STRSET (type,
+	                  NM_SETTING_VLAN_SETTING_NAME,
+	                  NM_SETTING_WIRELESS_SETTING_NAME,
+	                  NM_SETTING_INFINIBAND_SETTING_NAME,
+	                  NM_SETTING_BOND_SETTING_NAME,
+	                  NM_SETTING_TEAM_SETTING_NAME,
+	                  NM_SETTING_BRIDGE_SETTING_NAME))
+		return TRUE;
+	if (   nm_streq0 (type, NM_SETTING_WIRED_SETTING_NAME)
+	    && !nm_connection_get_setting_pppoe (connection))
 		return TRUE;
 
-	s_con = nm_connection_get_setting_connection (connection);
-	g_assert (s_con);
+	id = nm_connection_get_id (connection);
 	g_set_error (error, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_FAILED,
-	             "The ifcfg-rh plugin cannot write the connection '%s' (type '%s' pppoe %d)",
-	             nm_connection_get_id (connection),
-	             nm_setting_connection_get_connection_type (s_con),
-	             !!nm_connection_get_setting_pppoe (connection));
+	             "The ifcfg-rh plugin cannot write the connection %s%s%s (type %s%s%s)",
+	             NM_PRINT_FMT_QUOTE_STRING (id),
+	             NM_PRINT_FMT_QUOTE_STRING (type));
 	return FALSE;
 }
 
